﻿--[[----------------------------------------------------------------------
      VanasKoSGUI - Part of VanasKoS
Handles the main gui frame
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoS");
VanasKoSGUI = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0");

local dewdrop = AceLibrary("Dewdrop-2.0");

local listHandler = { };

function VanasKoSGUI:AddConfigOption(name, option)
	if(not self.ConfigurationOptions) then
		self.ConfigurationOptions = {
			type = 'group',
			args = {
			}
		}
	end
	
	self.ConfigurationOptions.args[name] = option;
end

function VanasKoSGUI:GetListName(listid)
	for k,v in pairs(VANASKOS.Lists) do
		if(v[1] == listid) then
			return v[2];
		end
	end
	return listid;
end

function VanasKoSGUI:OnInitialize()
	UIPanelWindows["VanasKoSFrame"] = { area = "left", pushable = 1, whileDead = 1 };

	local sortOptions = { 
		type = 'group',
		args = {
			byname = {
				name = L["by name"],
				desc = L["sort by name"],
				type = "toggle",
				isRadio = true,
				order = 1,
				get = function()
					if(VanasKoS:GetOpt("sortListBy") == "name") then
						return true;
					else
						return false;
					end
				end,
				set = function()
					VanasKoS:SetOpt("sortListBy", "name");
					VanasKoSGUI:ScrollFrameUpdate();
				end
			},
			bylevel = {
				name = L["by level"],
				desc = L["sort by level"],
				type = "toggle",
				isRadio = true,
				order = 2,
				get = function()
					if(VanasKoS:GetOpt("sortListBy") == "level") then
						return true;
					else
						return false;
					end
				end,
				set = function()
					VanasKoS:SetOpt("sortListBy", "level");
					VanasKoSGUI:ScrollFrameUpdate();
				end
			},
			byreason = {
				name = L["by reason"],
				desc = L["sort by reason"],
				type = "toggle",
				isRadio = true,
				order = 3,
				get = function()
					if(VanasKoS:GetOpt("sortListBy") == "reason") then
						return true;
					else
						return false;
					end
				end,
				set = function()
					VanasKoS:SetOpt("sortListBy", "reason");
					VanasKoSGUI:ScrollFrameUpdate();
				end
			},
			bylastseen = {
				name = L["by last seen"],
				desc = L["sort by last seen"],
				type = "toggle",
				isRadio = true,
				order = 4,
				get = function()
					if(VanasKoS:GetOpt("sortListBy") == "lastseen") then
						return true;
					else
						return false;
					end
				end,
				set = function()
					VanasKoS:SetOpt("sortListBy", "lastseen");
					VanasKoSGUI:ScrollFrameUpdate();
				end
			},
			bycreator = {
				name = L["by creator"],
				desc = L["sort by creator"],
				type = "toggle",
				isRadio = true,
				order = 5,
				get = function()
					if(VanasKoS:GetOpt("sortListBy") == "creator") then
						return true;
					else
						return false;
					end
				end,
				set = function()
					VanasKoS:SetOpt("sortListBy", "creator");
					VanasKoSGUI:ScrollFrameUpdate();
				end
			},
		}
	};
	
	VanasKoSListFrameSortButton:SetScript("OnClick", function()
			dewdrop:Register(this,
				'children', sortOptions,
				'point', "TOPLEFT",
				'dontHook', true,
				'relativePoint', "TOPRIGHT"
			);
			this:SetScript("OnClick", function()
				if(dewdrop:IsOpen(this)) then
					dewdrop:Close();
				else
					dewdrop:Open(this);
				end
			end);
			this:GetScript("OnClick")();
	end);
	
	VanasKoSListFrameConfigurationButton:SetScript("OnClick", function()
			dewdrop:Register(this, 
							'children', self.ConfigurationOptions,
							'point', "TOPLEFT",
							'dontHook', true,
							'relativePoint', "TOPRIGHT");
			this:SetScript("OnClick", function()
				if(dewdrop:IsOpen(this)) then
					dewdrop:Close();
				else
					dewdrop:Open(this);
				end
			end);
			this:GetScript("OnClick")();
	end);
	
	self:RegisterEvent("VanasKoS_List_Added", "InitializeDropDowns");
end

function VanasKoSGUI:InitializeDropDowns()
	UIDropDownMenu_Initialize(VanasKoSFrameChooseListDropDown, 
		function() 
			local lists = VANASKOS.Lists;
			local i = 0;
			for k,v in self:pairsByKeys(lists, nil) do
				local button = UIDropDownMenu_CreateInfo();
				button.text = v[2];
				button.value = v[1];
				button.func = function() 
						UIDropDownMenu_SetSelectedValue(VanasKoSFrameChooseListDropDown, this.value);
						VANASKOS.showList = this.value;
						VanasKoSGUI:ScrollFrameUpdate();
				end
				UIDropDownMenu_AddButton(button); 
				i = i + 1;
			end
	end);
	UIDropDownMenu_SetSelectedValue(VanasKoSFrameChooseListDropDown, "PLAYERKOS");
end

function VanasKoSGUI:OnEnable()
	self:RegisterEvent("VanasKoS_List_Entry_Added", function() VanasKoSGUI:ScrollFrameUpdate(); end);
	self:RegisterEvent("VanasKoS_List_Entry_Removed", function() VanasKoSGUI:ScrollFrameUpdate(); end);
end

function VanasKoSGUI:OnDisable()
end

--[[----------------------------------------------------------------------
		GUI Functions
------------------------------------------------------------------------]]

VANASKOS.selectedEntry = nil;

local VANASKOSFRAME_SUBFRAMES = { "VanasKoSListFrame", "VanasKoSAboutFrame" };

function VanasKoSGUI:GUIFrame_ShowSubFrame(frameName)
	for index, value in pairs(VANASKOSFRAME_SUBFRAMES) do
		if(value == frameName) then
			getglobal(value):Show();
		else
			getglobal(value):Hide();
		end
	end
end

function VanasKoSGUI:GUIShowChangeDialog()
	local dialog = nil;
	if(VANASKOS.selectedEntry) then
		dialog = StaticPopup_Show("VANASKOS_CHANGE_ENTRY");
		local name = self:GetSelectedEntryName();
		local reason = "";
		local list = self:GetCurrentShownList();
		if(VANASKOS.showList == "PLAYERKOS" or VANASKOS.showList == "GUILDKOS" or VANASKOS.showList == "HATELIST" or VANASKOS.showList == "NICELIST") then
			reason = list[string.lower(name)].reason;
		end
		getglobal(dialog:GetName() .. "EditBox"):SetText(reason);
	end
end

function VanasKoSGUI:GetCurrentShownList()
	return VanasKoS:GetList(VANASKOS.showList);
end

function VanasKoSGUI:GUIChangeKoSReason(reason)
	local name = self:GetSelectedEntryName();
	local list = self:GetCurrentShownList();
	list[string.lower(name)].reason = reason;
	self:ScrollFrameUpdate();
end

function VanasKoSGUI:GetSelectedEntryName()
	if(VANASKOS.selectedEntry) then
			local listVar = self:GetCurrentShownList();
			local listIndex = 1;
			
			if(listVar == nil) then
				self:GUIHideButtons(1, 10);
				return nil;
			end

			for k,v in self:SortedList(listVar) do
				if(listIndex == VANASKOS.selectedEntry) then
					return k;
				end
				listIndex = listIndex + 1;
			end

	end
	return nil;
end

function VanasKoSGUI:GetSelectedButtonNumber()
	if(VANASKOS.selectedEntry) then
			local listVar = self:GetCurrentShownList();
			local listIndex = 1;
			
			if(listVar == nil) then
				self:GUIHideButtons(1, 10);
				return nil;
			end

			for k,v in self:SortedList(listVar) do
				if(listIndex == VANASKOS.selectedEntry) then
					return listIndex;
				end
				listIndex = listIndex + 1;
			end

	end
	return nil;
end

function VanasKoSGUI:RemoveEntry()
	VanasKoS:RemoveEntry(VANASKOS.showList, VanasKoSGUI:GetSelectedEntryName());
end

function VanasKoSGUI:ListButton_OnClick(button)
	if(button == "LeftButton") then
		VANASKOS.selectedEntry = this:GetID();
		self:ScrollFrameUpdate();
	else
		VANASKOS.selectedEntry = this:GetID();
		self:ScrollFrameUpdate();
		VanasKoSSynchronizer:ShowContextMenuForSyncList();
	end
end

function VanasKoSGUI:GUIHideButtons(minimum, maximum)
	for i=minimum,maximum,1 do
		local button = getglobal("VanasKoSListFrameListButton" .. i);
		if(button ~= nil) then
			button:Hide();
		end
	end
end

function VanasKoSGUI:GUIShowButtons(minimum, maximum)
	for i=minimum,maximum,1 do
		local button = getglobal("VanasKoSListFrameListButton" .. i);
		button:Show();
	end
end

function VanasKoSGUI:pairsByKeys (t, f)
	local a = {}
	for n in pairs(t) do table.insert(a, n) end
	table.sort(a, f)
	local i = 0      -- iterator variable
	local iter = function ()   -- iterator function
		i = i + 1
		if a[i] == nil then 
			return nil
		else 
			return a[i], t[a[i]]
		end
	end
	return iter
end

function VanasKoSGUI:SortedList(list)
	local sortListBy = VanasKoS:GetOpt("sortListBy");
	local sortFunction = nil;
	
	if(VANASKOS.showList == "PLAYERKOS" or VANASKOS.showList == "HATELIST" or VANASKOS.showList == "NICELIST") then
		if(sortListBy == "level") then
			-- sort by level
			sortFunction = function(val1, val2) 
								local list = VanasKoS:GetList("PLAYERDATA");
								if(list ~= nil) then
									local cmp1 = 0;
									local cmp2 = 0;
									if(list[val1] ~= nil and list[val1].level ~= nil) then
										cmp1 = list[val1].level;
									end
									if(list[val2] ~= nil and list[val2].level ~= nil) then
										cmp2 = list[val2].level;
									end
									if(cmp1 < cmp2) then
										return true;
									else
										return false;
									end
								end
							end
		elseif(sortListBy == "lastseen") then
			-- sort by reason
			sortFunction = function(val1, val2) 
								local list = VanasKoS:GetList("PLAYERDATA");
								if(list ~= nil) then
									local cmp1 = 0;
									local cmp2 = 0;
									if(list[val1] ~= nil and list[val1].lastseen ~= nil) then
										cmp1 = time() - list[val1].lastseen;
									end
									if(list[val2] ~= nil and list[val2].lastseen ~= nil) then
										cmp2 = time() - list[val2].lastseen;
									end
									
									if(cmp1 < cmp2) then
										return true;
									else
										return false;
									end
								end
							end
		end
	end
	if(VANASKOS.showList == "PLAYERKOS" or 
		VANASKOS.showList == "GUILDKOS" or 
		VANASKOS.showList == "HATELIST" or 
		VANASKOS.showList == "NICELIST") then
		if(sortListBy == "reason") then
		-- sort by reason
			sortFunction = function(val1, val2)
					local list = VanasKoS:GetList(VANASKOS.showList);
					local str1 = list[val1].reason;
					local str2 = list[val2].reason;
					if(str1 == nil or str1 == "") then
						str1 = L["_Reason Unknown_"];
					end
					if(str2 == nil or str2 == "") then
						str2 = L["_Reason Unknown_"];
					end
					if(str1:lower() < str2:lower()) then
						return true;
					else
						return false;
					end
				end
		elseif(sortListBy == "creator") then
		-- sort by reason
			sortFunction = function(val1, val2)
					local list = VanasKoS:GetList(VANASKOS.showList);
					local str1 = list[val1].creator;
					local str2 = list[val2].creator;
					if(str1 == nil or str1 == "") then
						str1 = "";
					end
					if(str2 == nil or str2 == "") then
						str2 = "";
					end
					if(str1:lower() < str2:lower()) then
						return true;
					else
						return false;
					end
				end
		end
	end

	return self:pairsByKeys(list, sortFunction);
end

local oldlist = "";
function VanasKoSGUI:RegisterList(listName, handlerObject)
	if(not handlerObject) then
		VanasKoS:Print("Error: No handlerObject for " .. listName);
		return;
	end
	if(not handlerObject.RenderButton) then
		VanasKoS:Print("Error: No RenderButton method for " .. listName);
		return;
	end
	listHandler[listName] = handlerObject;
end

function VanasKoSGUI:ScrollFrameUpdate()
	local listOffset = FauxScrollFrame_GetOffset(VanasKoSListScrollFrame);
	local listVar = self:GetCurrentShownList();
	local listIndex = 1;
	local buttonIndex = 1;

	if(listVar == nil) then
		VANASKOS.selectedEntry = nil;
		self:GUIHideButtons(1, 10);
		return nil;
	end
	
	-- stupid fix that the dropdown selection is correct if showList is set external
	UIDropDownMenu_SetSelectedValue(VanasKoSFrameChooseListDropDown, VANASKOS.showList);

	if(oldlist ~= VANASKOS.showList) then
		if(listHandler[oldlist] and listHandler[oldlist].HideList) then
			listHandler[oldlist]:HideList(oldlist);
		end
		if(listHandler[VANASKOS.showList] and listHandler[VANASKOS.showList].ShowList) then
			listHandler[VANASKOS.showList]:ShowList(VANASKOS.showList);
		end
	end
	
	for k,v in self:SortedList(listVar) do
 		if((listIndex-1) < listOffset) then
		else
			if(buttonIndex <= 10) then
				local buttonText1 = getglobal("VanasKoSListFrameListButton" ..  buttonIndex .. "ButtonTextName");
				local buttonText2 = getglobal("VanasKoSListFrameListButton" ..  buttonIndex .. "ButtonTextReason");
				local button = getglobal("VanasKoSListFrameListButton" .. buttonIndex);
				button:SetID(listIndex);
				if(listIndex == VANASKOS.selectedEntry) then
					button:LockHighlight();
				else
					button:UnlockHighlight();
				end
				
				if(listHandler[VANASKOS.showList] ~= nil) then
					listHandler[VANASKOS.showList]:RenderButton(VANASKOS.showList, buttonIndex, button, k, v, buttonText1, buttonText2);
				end
				
				buttonIndex = buttonIndex + 1;
			end
		end
		listIndex = listIndex + 1;
	end
	
	if(listIndex <= 10) then
		self:GUIHideButtons(listIndex, 10);
	end
	
	if(listIndex == 1) then
		VANASKOS.selectedEntry = nil;
	else
		if(VANASKOS.selectedEntry == nil) then
			VANASKOS.selectedEntry = 1;
			self:ScrollFrameUpdate();
		end
		if(VANASKOS.selectedEntry >= listIndex) then
			VANASKOS.selectedEntry = 1;
			self:ScrollFrameUpdate();
		end
	end

	-- 34 = Hoehe VanasKoSListFrameListButtonTemplate
	-- scrollframe, maxnum, to_display, height
	FauxScrollFrame_Update(VanasKoSListScrollFrame, listIndex-1, 10, 34);
	
	oldlist = VANASKOS.showList;
end

function VanasKoSGUI:AddEntry()
	if(VANASKOS.showList == "GUILDKOS") then
		VANASKOS.LastNameEntered = GetGuildInfo("target");
	elseif(VANASKOS.showList == "SYNCPLAYER") then
		VANASKOS.LastNameEntered = nil;
	else
		VANASKOS.LastNameEntered = UnitName("target");
	end
	if(VANASKOS.LastNameEntered) then
		if(UnitIsPlayer("target")) then
			StaticPopup_Show("VANASKOS_ADD_REASON_ENTRY");
		else
			StaticPopup_Show("VANASKOS_ADD_ENTRY");
		end
	else
		StaticPopup_Show("VANASKOS_ADD_ENTRY");
	end
end

function VanasKoS_GUIOnScroll()
	VanasKoSGUI:ScrollFrameUpdate();
end

function VanasKoSGUI:Update()
	if(not VanasKoSFrame:IsVisible()) then
		return nil;
	end
	
	if(VanasKoSFrame.selectedTab == 1) then
		VanasKoSFrameTopLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopLeft");
		VanasKoSFrameTopRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
		VanasKoSFrameBottomLeft:SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-BotLeft");
		VanasKoSFrameBottomRight:SetTexture("Interface\\FriendsFrame\\UI-FriendsFrame-BotRight");
		
		VanasKoSFrameTitleText:SetText(VANASKOS.NAME .. " - " .. L["Lists"]);
		self:GUIFrame_ShowSubFrame("VanasKoSListFrame");
		self:ScrollFrameUpdate();
	end
	if(VanasKoSFrame.selectedTab == 2) then
		VanasKoSFrameTopLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopLeft");
		VanasKoSFrameTopRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
		VanasKoSFrameBottomLeft:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-BottomLeft");
		VanasKoSFrameBottomRight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-BottomRight");

		self:GUIFrame_ShowSubFrame("VanasKoSAboutFrame");
		VanasKoSFrameTitleText:SetText(VANASKOS.NAME .. " - " .. L["About"]);
	end
end

function string.Capitalize( str )
    if(GetLocale() == "koKR") then return str end
    return string.upper( string.sub( str, 1, 1 ) )..string.sub( str, 2 )
end

--[[---------------------------------------------------------------------------------------------------
				Popup Dialog
-----------------------------------------------------------------------------------------------------]]
StaticPopupDialogs["VANASKOS_ADD_REASON_ENTRY"] = {
	text = L["Reason"],
	button1 = L["Accept"],
	button2 = L["Cancel"],
	hasEditBox = 1,
	maxLetters = 40,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		local reason = editBox:GetText();
		if(reason == nil or reason == "") then
			reason = L["_Reason Unknown_"];
		end
		VanasKoS:AddEntry(VANASKOS.showList, VANASKOS.LastNameEntered, { ['reason'] = reason });
		VanasKoSGUI:Update();
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		local reason = editBox:GetText();
		if(reason == nil or reason == "") then
			reason = L["_Reason Unknown_"];
		end
		VanasKoS:AddEntry(VANASKOS.showList, VANASKOS.LastNameEntered, { ['reason'] = reason });
		VanasKoSGUI:Update();
		this:GetParent():Hide();
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnHide = function()
		if(ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["VANASKOS_ADD_ENTRY"] = {
	text = L["Name"],
	button1 = L["Accept"],
	button2 = L["Cancel"],
	hasEditBox = 1,
	maxLetters = 40,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		VANASKOS.LastNameEntered = editBox:GetText();
		if(VANASKOS.showList == "SYNCPLAYER") then
			VanasKoS:AddEntry(VANASKOS.showList, VANASKOS.LastNameEntered,  { ['reason'] = "" });
			return;
		end
		if(VANASKOS.LastNameEntered ~= "") then
			StaticPopup_Show("VANASKOS_ADD_REASON_ENTRY");
		end
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		VANASKOS.LastNameEntered = editBox:GetText();
		if(VANASKOS.showList == "SYNCPLAYER") then
			VanasKoS:AddEntry(VANASKOS.showList, VANASKOS.LastNameEntered,  { ['reason'] = "" });
			this:GetParent():Hide();
			return;
		end
		if(VANASKOS.LastNameEntered ~= "") then
			StaticPopup_Show("VANASKOS_ADD_REASON_ENTRY");
		end
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetText("");
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnHide = function()
		if(ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}

StaticPopupDialogs["VANASKOS_CHANGE_ENTRY"] = {
	text = L["Reason"],
	button1 = L["Accept"],
	button2 = L["Cancel"],
	hasEditBox = 1,
	maxLetters = 40,
	OnAccept = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		VanasKoSGUI:GUIChangeKoSReason(editBox:GetText());
	end,
	EditBoxOnEnterPressed = function()
		local editBox = getglobal(this:GetParent():GetName().."EditBox");
		VanasKoSGUI:GUIChangeKoSReason(editBox:GetText());
		this:GetParent():Hide();
	end,
	OnShow = function()
		getglobal(this:GetName().."EditBox"):SetFocus();
	end,
	OnHide = function()
		if(ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
		getglobal(this:GetName().."EditBox"):SetText("");
	end,
	EditBoxOnEscapePressed = function()
		this:GetParent():Hide();
	end,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
	hideOnEscape = 1
}
