--[[----------------------------------------------------------------------
      DistributedTracker Module - Part of VanasKoS
Broadcasts a list on ZONE and GUILD and handles returning position answers
------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSDistributedTracker");

VanasKoSTracker = VanasKoS:NewModule("DistributedTracker");

L:RegisterTranslations("enUS", function() return {
	["Position on Player %s (%d, %d in %s) received from %s"] = true,
	["%s in %s at %d, %d (%s) - Create Cartographer Note?"] = true,
	["Ok"] = true,
	["Cancel"] = true,
	["Distributed Tracking"] = true,
	["Enabled"] = true,
} end);

L:RegisterTranslations("deDE", function() return {
	["Position on Player %s (%d, %d in %s) received from %s"] = "Position von Player %s empfangen (%d, %d in %s) von %s",
	["%s in %s at %d, %d (%s) - Create Cartographer Note?"] = "%%s in %s bei %d, %d (%s) - Cartographer Notiz erstellen?",
	["Ok"] = "Ok",
	["Cancel"] = "Abbrechen",
	["Distributed Tracking"] = "Verteiltes Suchen",
	["Enabled"] = "Aktiviert",
} end);

L:RegisterTranslations("frFR", function() return {
	["Position on Player %s (%d, %d in %s) received from %s"] = "Position du joueur %s (%d, %d \195\128 %s) re\195\167u de %s",
	["%s in %s at %d, %d (%s) - Create Cartographer Note?"] = "%s in %s at %d, %d (%s) - Cr\195\169er une note avec \"Cartographer\" ?",
	["Ok"] = "Ok",
	["Cancel"] = "Annuler",
	["Distributed Tracking"] = "R\195\169partition des informations KoS",
	["Enabled"] = "Actif",
} end);

L:RegisterTranslations("koKR", function() return {
--	["Position on Player %s (%d, %d in %s) received from %s"] = true,
--	["%s in %s at %d, %d (%s) - Create Cartographer Note?"] = true,
	["Ok"] = "확인",
	["Cancel"] = "취소",
	["Distributed Tracking"] = "분산된 추적",
	["Enabled"] = "사용중",
} end);

-- don't change the values!
-- time span for announcing the kos list
local PUBLISH_REPEAT_TIME = 600;
-- time span after a koslist was received, until the kos-list gets purged
local LIST_PURGE_TIME = 660;
-- time until a new location for player A that was sent to B can be sent again
local STOP_REPORTING_DELAY = 800;

--[[
	Messaging format for Grudgebook for later usage:
		Channel: GrugeBookSync
		Request for Data: GBLSREQ!<name>
		Response: GBLSRES!<name>:<guild>:<location>:<time>:<player>
--]]
local watchList = { };
local blockedList = { };
local zoneNames = nil;

function VanasKoSTracker:OnInitialize()
	self:SetCommPrefix("VanasKoSTracker");
	self:SetDefaultCommPriority("BULK");
	
	VanasKoSGUI:AddConfigOption("DistributedTracking",
		{
			type = 'group',
			name = L["Distributed Tracking"],
			desc = L["Distributed Tracking"],
			args = {
				enabled = {
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled"],
					set = function() VanasKoS:TogOpt("DistributedTracking"); VanasKoS:ToggleModuleActive("DistributedTracker", VanasKoS:GetOpt("DistributedTracking")); end,
					get = function() return VanasKoS:GetOpt("DistributedTracking"); end,
				}
			}
		});
end

function VanasKoSTracker:OnEnable()
	if(not VanasKoS:GetOpt("DistributedTracking")) then
		return;
	end
	
	self:RegisterComm(self.commPrefix, "ZONE", "ListMessageReceived");
	self:RegisterComm(self.commPrefix, "WHISPER", "PositionUpdateReceived");
	self:RegisterComm(self.commPrefix, "GUILD", "ListMessageReceived");
	
	self:RegisterEvent("VanasKoS_Player_Detected", "Player_Detected");
	self:ScheduleRepeatingEvent(self.PublishList, PUBLISH_REPEAT_TIME, self);
end

function VanasKoSTracker:OnDisable()
	-- disabled due to buggyness
	--self:UnregisterAllComms();
	self:CancelAllScheduledEvents(); 
end

function VanasKoSTracker:ListMessageReceived(prefix, sender, distribution, playerlist)
	watchList[sender] = playerlist;
	
	-- id must be given to prevent execution of ListPurge if new list came
	self:ScheduleEvent("ListPurge" .. sender, self.PurgeListData, LIST_PURGE_TIME, self, sender);
end

function VanasKoSTracker:PurgeListData(name)
	watchList[name] = nil;
end

function VanasKoSTracker:PositionUpdateReceived(prefix, sender, distribution, playername, posX, posY, continent, zone)
	zoneNames = { GetMapZones(continent) };

	-- continent = 3 and zone = 6 => shattrah city
	if(continent ~= 3 and zone ~= 6) then
		return;
	end
	
	VanasKoS:Print(format("Position on Player %s (%d, %d in %s) received from %s", playername, posX*100, posY*100, zoneNames[zone], sender));
	if(not Cartographer_Notes) then
		return;
	end
	
	local dialog = StaticPopup_Show("VANASKOS_TRACKER_ADD_NOTE");
	if(dialog) then
		dialog.kossender = sender;
		dialog.kosname = playername;
		dialog.koszone = zoneNames[zone];
		dialog.kosPosX = posX;
		dialog.kosPosY = posY;

		getglobal(dialog:GetName() .. "Text"):SetText(format(L["%s in %s at %d, %d (%s) - Create Cartographer Note?"], playername, zoneNames[zone], posX*100, posY*100, sender));	end
end

function VanasKoSTracker:AddToBlockedList(kosTarget, playername)
	if(self:IsOnBlockedList(kosTarget, searchingPlayer)) then
		return;
	end
	local list = blockedList[kosTarget];
	if(list == nil) then
		list = { };
	end
	tinsert(list, playername);
	blockedList[kosTarget] = list;
end

function VanasKoSTracker:Player_Detected(locatedPlayer, guild, faction)
	if(locatedPlayer == nil) then
		return;
	end
-- for debuggout comment out
	if(faction == nil or locatedPlayer == nil or faction ~= "enemy") then
		return;
	end
	
	for name, list in pairs(watchList) do
		for index, playername in pairs(list) do
			if(playername:lower() == locatedPlayer:lower()) then
				if(not self:IsOnBlockedList(playername, name)) then
					-- add to blocked list, add event to remove him
					self:AddToBlockedList(playername, name);
					self:ScheduleEvent(self.RemoveFromBlocked, STOP_REPORTING_DELAY, self, playername, name);

					-- send position
					local posX, posY = GetPlayerMapPosition("player");
					local continent = GetCurrentMapContinent();
					local zone = GetCurrentMapZone();
					-- continent = 3 and zone = 6 => shattrah city
					if(continent ~= 3 and zone ~= 6) then
						self:SendCommMessage("WHISPER", name, playername, posX, posY, continent, zone);
					end
				end
			end
		end
	end
end

function VanasKoSTracker:RemoveFromBlocked(kosTarget, searchingPlayer)
	local list = blockedList[kosTarget];
	for k, v in pairs(list) do
		if(v == searchingPlayer) then
			tremove(list, k);
			blockedList[kosTarget] = list;
			return;
		end
	end
end

function VanasKoSTracker:IsOnBlockedList(kosTarget, searchingPlayer)
	local list = blockedList[kosTarget];
	if(list == nil) then
		return false;
	end
	
	for k, v in pairs(list) do
		if(v == searchingPlayer) then
			return true;
		end
	end
	
	return false;
end

function VanasKoSTracker:PrintWatchList()
	for playername,list in pairs(watchList) do
		VanasKoS:Print("Player:", playername);
		local msg = "";
		for index, kosname in pairs(list) do
			msg = msg .. " " .. kosname;
		end
		VanasKoS:Print(msg);
	end
	VanasKoS:Print("end");
end

function VanasKoSTracker:PrintBlockList()
	for playername,list in pairs(blockedList) do
		VanasKoS:Print("Player:", playername);
		for index, kosname in pairs(list) do
			VanasKoS:Print(" ", kosname);
		end
	end
	VanasKoS:Print("end");
end

function VanasKoSTracker:PublishList()
	local list = { }
	for k,v in pairs(VanasKoS:GetList("PLAYERKOS")) do
		if(not v.creator or v.creator == "" or v.creator == UnitName("player")) then
			tinsert(list, k);
		end
	end
	if(#list == 0) then
		return;
	end
	
	--debug
	--self:ListMessageReceived("moep", "Vane", "ZONE", list);
	
	self:SendCommMessage("GUILD", list); 
	self:SendCommMessage("ZONE", list);
end

StaticPopupDialogs["VANASKOS_TRACKER_ADD_NOTE"] = {
	text = "TEMPLATE",
	button1 = L["Ok"],
	button2 = L["Cancel"],
	OnAccept = function()
		if(Cartographer_Notes ~= nil) then
			local dialog = this:GetParent();
			Cartographer_Notes:SetNote(dialog.koszone, dialog.kosPosX, dialog.kosPosY, dialog.kosname, "VanasKoS-" .. dialog.kossender)
			if(Cartographer_Waypoints ~= nil) then
				Cartographer_Waypoints:SetNoteAsWaypoint(dialog.koszone, Cartographer_Notes.getID(dialog.kosPosX, dialog.kosPosY));
			end
		end
	end,
	OnShow = function(playername)
	end,
	OnHide = function()
		if(ChatFrameEditBox:IsVisible() ) then
			ChatFrameEditBox:SetFocus();
		end
	end,
	timeout = 0,
	exclusive = 0,
	whileDead = 1,
	hideOnEscape = 1
}