﻿--[[---------------------------------------------------------------------------------------------
      PvPDataGatherer Module - Part of VanasKoS
Gathers PvP Wins and Losses

Credits: DeuceLog for code on how to use Parser-3.0
---------------------------------------------------------------------------------------------]]

local L = AceLibrary("AceLocale-2.2"):new("VanasKoSPvPDataGatherer");

VanasKoSPvPDataGatherer = VanasKoS:NewModule("PvPDataGatherer");

local tourist = AceLibrary("Tourist-2.0")

L:RegisterTranslations("enUS", function() return {
	["PvP Data Gathering"] = true,
	["Enabled"] = true,
	["PvP Stats"] = true,
	["wins: %d - losses: %d"] = "wins: |cff00ff00%d|r losses: |cffff0000%d|r",
} end);

L:RegisterTranslations("deDE", function() return {
	["PvP Data Gathering"] = "PvP Daten sammeln",
	["Enabled"] = "Aktiviert",
	["PvP Stats"] = "PvP Statistik",
	["wins: %d - losses: %d"] = "gewonnen: |cff00ff00%d|r verloren: |cffff0000%d|r",
} end);

L:RegisterTranslations("frFR", function() return {
	["PvP Data Gathering"] = "Rassemblement de donn\195\169es PvP",
	["Enabled"] = "actif",
	["PvP Stats"] = "PvP Stats",
	["wins: %d - losses: %d"] = "victoires: |cff00ff00%d|r d\195\169faites: |cffff0000%d|r",
} end);

L:RegisterTranslations("koKR", function() return {
	["PvP Data Gathering"] = "PvP 데이터 수집",
	["Enabled"] = "사용중",
	["PvP Stats"] = "PvP 현황",
	["wins: %d - losses: %d"] = "승: |cff00ff00%d|r 패: |cffff0000%d|r",
} end);


function VanasKoSPvPDataGatherer:OnInitialize()
	self.db = VanasKoS.db;
	
	VanasKoSGUI:AddConfigOption("PvP Data Gathering",
		{
			type = 'group',
			name = L["PvP Data Gathering"],
			desc = L["PvP Data Gathering"],
			args = {
				enabled = {
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enabled"],
					order = 1,
					set = function() VanasKoS:TogOpt("PvPDataGathering"); VanasKoS:ToggleModuleActive("PvPDataGatherer", VanasKoS:GetOpt("PvPDataGathering")); end,
					get = function() return VanasKoS:GetOpt("PvPDataGathering"); end,
				},
			}
		});
	
	-- should be moved to pvpdatagatherer
	VanasKoS:RegisterList(5, "PVPSTATS", L["PvP Stats"], self);
	VanasKoSGUI:RegisterList("PVPSTATS", self);
end

function VanasKoSPvPDataGatherer:RenderButton(list, buttonIndex, button, key, value, buttonText1, buttonText2)
	if(list == "PVPSTATS") then
		buttonText1:SetText(string.Capitalize(key));
		local wins, losses = VanasKoS:IsOnList("PVPSTATS", key);
		if(wins == nil) then
			wins = 0;
		end
		if(losses == nil) then
			losses = 0;
		end
		buttonText2:SetText(format(L["wins: %d - losses: %d"], wins, losses));

		button:Show();
	end
end

function VanasKoSPvPDataGatherer:ShowList(list)
	VanasKoSListFrameSyncButton:Disable();
	VanasKoSListFrameChangeButton:Disable();
	VanasKoSListFrameAddButton:Disable();
end

function VanasKoSPvPDataGatherer:HideList(list)
	VanasKoSListFrameSyncButton:Enable();
	VanasKoSListFrameChangeButton:Enable();
	VanasKoSListFrameAddButton:Enable();
end

function VanasKoSPvPDataGatherer:GetList(list)
	if(list == "PVPSTATS") then
		return self.db.realm.pvpstats.players;
	else
		return nil;
	end
end

function VanasKoSPvPDataGatherer:AddEntry(list, name, data)
	return true;
end

function VanasKoSPvPDataGatherer:RemoveEntry(listname, name, guild)
	local list = self:GetList(listname);
	if(list and list[name]) then
		list[name] = nil;
		self:TriggerEvent("VanasKoS_List_Entry_Removed", listname, name);
	end
end

function VanasKoSPvPDataGatherer:IsOnList(list, name, guild)
	local listVar = self:GetList(list);
	if(list == "PVPSTATS") then
		if(listVar[name]) then
			return listVar[name].wins, listVar[name].losses;
		else
			return nil;
		end
	else
		return nil;
	end
end

function VanasKoSPvPDataGatherer:OnDisable()
	self:UnregisterAllParserEvents();
end

function VanasKoSPvPDataGatherer:OnEnable()
	if(not VanasKoS:GetOpt("PvPDataGathering")) then
		return;
	end
	self:RegisterParserEvent({
		eventType = "Damage",
		sourceID = "player",
		recipientPvP = true,
	}, "DamageDoneTo");
	
	self:RegisterParserEvent({
		eventType = "Damage",
		recipientID = "player",
		sourcePvP = true,
	}, "DamageDoneFrom");
	
	self:RegisterParserEvent({
		eventType = "Notification",
		notificationType = "Killing blow",
	}, "KillingBlow");

	self:RegisterParserEvent({
		eventType = "Notification",
		notificationType = "Honor gain",
	}, "HonorGain");

	self:RegisterEvent("PLAYER_DEAD");
	
	self:RegisterEvent("VanasKoS_PvP_Win", "LogPvPWin");
	self:RegisterEvent("VanasKoS_PvP_Loss", "LogPvPLoss");
	self:RegisterEvent("VanasKoS_Zone_Changed", "Zone_Changed");
end

local lastDamageTo = nil;
local lastDamageFrom = nil;

local inBattleground = false;

function VanasKoSPvPDataGatherer:LogPvPLoss(name)
	if(name == nil) then
		return;
	end
	name = name:lower();
--	self:Print("PvPLoss:", name);

	local list = VanasKoS:GetList("PVPSTATS");

	if(list[name] == nil) then
		list[name] = { };
	end

	local wins, losses = VanasKoS:IsOnList("PVPSTATS", name);
	if(losses == nil) then
		losses = 0;
	end
	losses = losses + 1;
	list[name].losses = losses
end

function VanasKoSPvPDataGatherer:LogPvPWin(name)
	if(name == nil) then
		return;
	end
	name = name:lower();
--	self:Print("PvPWin:", name);
	local list = VanasKoS:GetList("PVPSTATS");

	if(list[name] == nil) then
		list[name] = { };
	end

	local wins, losses = VanasKoS:IsOnList("PVPSTATS", name);
	if(wins == nil) then
		wins = 0;
	end
	wins = wins + 1;
	list[name].wins = wins
end

function VanasKoSPvPDataGatherer:DamageDoneFrom(t)
	if(not inBattleground) then
 		lastDamageFrom = t.sourceName;
	end
end

function VanasKoSPvPDataGatherer:DamageDoneTo(t)
	if(not inBattleground) then
		lastDamageTo = t.recipientName;
		self:RegisterEvent("ResetDamage", "ResetDamageTo", 5);
	end
end

function VanasKoSPvPDataGatherer:ResetDamageTo()
	lastDamageTo = nil;
end

function VanasKoSPvPDataGatherer:HonorGain(t)
	if(not inBattleground and lastDamageTo ~= nil) then
		lastDamageTo = nil;
		-- don't count honor gains when in raid/party, count killingblow instead (only reported when inside party/raid)
		if(GetNumPartyMembers() ~= 0 or UnitInRaid("player")) then
			return;
		end
		self:TriggerEvent("VanasKoS_PvP_Win", lastDamageTo);
	end
end

function VanasKoSPvPDataGatherer:PLAYER_DEAD(name)
	if(not lastDamageFrom or lastDamageFrom == "") then
		if(not name or name == "") then
			return;
		end
		lastDamageFrom = name;
	end
	if(not inBattleground) then
		self:TriggerEvent("VanasKoS_PvP_Loss", lastDamageFrom);
		lastDamageFrom = nil;
	end
end

function VanasKoSPvPDataGatherer:KillingBlow(t)
	-- exclude battleground
	if(not inBattleground  and lastDamageTo ~= nil) then
		self:TriggerEvent("VanasKoS_PvP_Win", lastDamageTo);
		lastDamageTo = nil;
	end
end

function VanasKoSPvPDataGatherer:Zone_Changed(zonetext)
	if(tourist:IsBattleground(zonetext) or tourist:IsInstance(zonetext) or tourist:IsArena(zonetext)) then
		inBattleground = true;
	else
		inBattleground = false;
	end
end